from sublime import *
from sublime_plugin import *
from os.path import exists, join, dirname
from os import mkdir
import re
import os
import subprocess

#globals
default_settings = r"""game_path: 'C:\Program Files (x86)\Steam\steamapps\common\Crusader Kings II'
mods_path: 'C:\Users\pegnu\Documents\Paradox Interactive\Crusader Kings II\mod'
font_face: 'Source Code Pro'"""
game_path_re	= re.compile(r"^\s*game_path:\s*'([^']*)'\s*$")
mods_path_re 	= re.compile(r"^\s*mods_path:\s*'([^']*)'\s*$")
font_face_re 	= re.compile(r"^\s*font_face:\s*'([^']*)'\s*$")
game_path = None
mods_path = None
font_face = None
game_path_found = False
mods_path_found = False
font_face_found = False
level_0_folders = ['events', 'decisions', 'music', 'localisation' ]
common_subfolders = ['artifact_spawns', 'artifacts', 'bookmarks', 'buildings', 'cb_types', 'combat_tactics', 'council_positions',
	'council_voting', 'cultures', 'death', 'death_text', 'disease', 'dynasties', 'event_modifiers', 'execution_methods',
	'game_rule', 'goernment_flavors', 'governments', 'graphicalculturetypes', 'heir_text', 'holding_types', 'job_actions',
	'job_titles', 'landed_titles', 'laws', 'mercenaries', 'minor_titles', 'modifier_definitions', 'nicknames', 'objectives', 
	'offmap_powers', 'on_actions', 'opinion_modifiers', 'religion_modifiers', 'religions', 'religious_titles', 'retinue_subunits'
	'scripted_triggers', 'scripted_effects', 'societies', 'special_troops', 'trade_routes', 'traits', 'tributary_tyoes', 
	'triggered_modifiers' ]
history_subfolders = ['technology', 'wars', 'titles', 'provinces', 'offmap_powers', 'characters']
views = {}
#plugin methods
def plugin_loaded():
	load_ck2ide_settings()
#helper functions
def common_prefix(a,b):
	a = a.split(os.sep); b = b.split(os.sep)
	r = range( min( len(a), len(b) ) )
	return os.sep.join([a[i] for i in r if a[i] == b[i]])
def parse_ck2ide_settings(path):
	global game_path, mods_path, font_face, game_path_found, mods_path_found, font_face_found
	if os.access(path, os.R_OK):
		with open(path, 'r') as f:
			for line in f:
				if not game_path_found:
					m = game_path_re.match(line)
					if 	 m: game_path = m.group(1); game_path_found = True
				elif not mods_path_found:
					m = mods_path_re.match(line)
					if 	 m: mods_path = m.group(1); mods_path_found = True
				elif not font_face_found:
					m = font_face_re.match(line)
					if 	 m: font_face = m.group(1); font_face_found = True
	else: print('The path "' + path + '" is not accessible or does not exist')
def load_ck2ide_settings():
	global game_path, mods_path, font_face, game_path_found, mods_path_found, font_face_found
	path = join(packages_path(), 'User', 'ck2IDE', 'settings.CK2IDE')
	if os.access(path, os.R_OK): 
		with open(path, 'r') as f:
			for line in f:
				if not game_path_found:
					m = game_path_re.match(line)
					if 	 m: game_path = m.group(1); game_path_found = True
				elif not mods_path_found:
					m = mods_path_re.match(line)
					if 	 m: mods_path = m.group(1); mods_path_found = True
				elif not font_face_found:
					m = font_face_re.match(line)
					if 	 m: font_face = m.group(1); font_face_found = True
	else: #since settings doesn't exists, we should re-create them.
		mkdir(dirname(path))
		with open(path, 'w+') as f: f.write(default_settings)
def apply_ck2ide_syntax(view):
		global game_path, mods_path, font_face, views
		views[view.id()] = False
		path = view.file_name()
		common = common_prefix(path, game_path); 
		if exists(common): path = path[len(common)+1:].split(os.sep)
		else: common = common_prefix(path, mods_path); path = path[len(common)+1:].split(os.sep)[1:]
		if not exists(common): print('file is not a ck2 game file'); return
		print(path)
		view.settings().set('font_face', font_face)
		view.settings().set("color_scheme", "Packages/ck2IDE/color_schemes/Monokai - ck2.tmTheme")
		if path[-1].endswith('.csv'):
			if len(path) == 2 and path[0] == 'localisation':
				view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/localisation/localisation.sublime-syntax')
				views[view.id()] = True
		elif path[-1].endswith('.txt'):
			if len(path) == 2:
				if path[0] in level_0_folders:
					view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/{}.sublime-syntax'.format(path[0]))
					views[view.id()] = True
				elif path[0] == 'common' and path[1] == 'technology.txt':
					view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/common/technology.sublime-syntax')
					views[view.id()] = True
			elif len(path) == 3:
				if path[0] == 'common' and path[1] in common_subfolders:
					view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/common/{}.sublime-syntax'.format(path[1]))
					views[view.id()] = True
				if path[0] == 'history' and path[1] in history_subfolders:
					view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/history/{}.sublime-syntax'.format(path[1]))
					views[view.id()] = True
				if path[0] == 'localisation' and path[1] == 'customizable_localisation':
					view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/localisation/customizable_localisation.sublime-syntax')
					views[view.id()] = True
			elif len(path) == 4:
				if path[0] == 'common' and path[1] == 'offmap_powers' and ( path[2] == 'policies' or path[2] == 'statuses'):
					view.set_syntax_file('Packages/ck2IDE/syntaxes/DSL/common/offmap_status_and_policies.sublime-syntax')
					views[view.id()] = True
def void(void): pass
#commands
class show_settings(TextCommand):
	def description(self):
		return "Settings"
	def run(self, edit):
		self.view.window().open_file(join(packages_path(), 'User', 'ck2IDE', 'settings.CK2IDE'))
class restore_settings(TextCommand):
	def description(self):
		return "Restore settings"
	def run(self, edit):
		path = join(packages_path(), 'User', 'ck2IDE', 'settings.CK2IDE')
		with open(path, 'w+') as f: f.write(default_settings)
		self.view.window().open_file(path)
class find_errors(TextCommand):
	def description(self):
		return "ck2IDE: Go to next error"
	def is_visible(self):
		global views
		if views[self.view.id()]: return True
		else: return False
	def is_enabled(self):
		if len(self.view.find_by_selector('ck2.illegal')) > 0: return True
		else: return False
	def run(self, edit):
		v = self.view
		invalid_regions = v.find_by_selector('ck2.illegal')
		if len(invalid_regions) > 0: 
			for x in invalid_regions:
				if x.intersects(v.visible_region()): continue
				else: v.show_at_center(x)
#event listeners
class ck2IDE_event_listener(ViewEventListener):
	def on_new(self):
		apply_ck2ide_syntax(self.view)
	def on_clone(self):
		apply_ck2ide_syntax(self.view)
	def on_load(self):
		apply_ck2ide_syntax(self.view)
	def on_post_save(self):
		apply_ck2ide_syntax(self.view)
	def on_close(self):
		global views
		views.pop(self.view.id(), None)
#END